import React, { useEffect, useState, useMemo } from 'react'
import { Select, message } from 'antd'
import { GetSelectItemList, GetFieldValueFromTable, GetFieldValueByTableName } from '../../../../apis/process'
import eventEmitter from '../../../../utils/eventEmitter';
const { Option } = Select

const RelevanceSelect = (props) => {

  const { value, onChange, schema, addons, title } = props
  const { $id, disabled, placeholder, fieldParent, dictionary, presetValue, whereField, whereType, sourceType, fieldTableKey, tableName, fieldName, displayName } = schema
  const [options, setOptions] = useState([])
  const [tableData, setTableData] = useState([])  

  //用于表数据的父子级控件之间联动
  useEffect(() => {
    if(getTargetPath()){
      const path = getTargetPath()
      eventEmitter.on(`${path}_event`, (val)=> dealParentData(val))
    }
  
    return () => {
      if(getTargetPath()){
        const path = getTargetPath(fieldTableKey)
        eventEmitter.removeAllListeners(`${path}_event`);
      }
    }
  }, [addons])

  useEffect(() => {
    if(sourceType === '表数据' && presetValue){
      setTimeout(() => {
        getTableData()
      }, 100);
    }
    if (addons) {
      addons.setValue(addons.dataPath, presetValue || '')
      if(!fieldTableKey && getChildrenSchemaByKey(title)){//表数据类型的父级控件联动子级加载数据
          eventEmitter.emit(`${addons?.dataPath}_event`, presetValue || '')
      }
    } else {
      onChange(presetValue)
    }

  }, [presetValue, sourceType])

  const selectChange = (value) => {
    if (addons) {
      onChange(value || '')
      
      if(getChildrenSchemaByKey(title)){
        eventEmitter.emit(`${addons?.dataPath}_event`, value)
      }
    }
  }

  ///处理父级传入的数据-表数据
  const dealParentData = async (val) => {
    console.log(val, '父级数据');
    const { code, data, msg } = await GetFieldValueByTableName({
      params: {
        tableName: tableName,
        fieldName: `${displayName},${fieldName}`,
      },
      data: [{
        field: whereField,
        type: whereType,
        value: val
      }]
    })
    if (code === 0) {
      if (Array.isArray(data)) {
        setTableData(data.filter(v => v))
      } else {
        setTableData([])
      }
    } else {
      message.error(msg)
    }
  }

  const getTargetPath = () => {
    if(sourceType != '表数据' || !fieldTableKey || !addons) return null;
    let targetPath = null
    for(const k of Object.keys(addons.formData)){
      for(const s of Object.keys(addons.formData[k])){
        if(fieldTableKey === s){
          targetPath = `${k}.${s}`
          break;
        }
    }
  }
    return targetPath
  }

  const getChildrenSchemaByKey = (key) => {
    if(sourceType != '表数据') return null;
    let schema = null;
    for(const k of Object.keys(addons.formData)){
        for(const s of Object.keys(addons.formData[k])){
          const path = `${k}.${s}`
          let _schema = addons?.getSchemaByPath(path)
          if(_schema?.fieldTableKey === key){
            schema = _schema
            break;
          }
      }
    }
    return schema
  }

  const enums = useMemo(() => {
    switch (sourceType) {
      case '数据字典':
        return Array.isArray(options) ? options : []
      case '表数据':
        return Array.isArray(tableData) ? tableData : []
      default:
        return []
    }
  }, [sourceType, options, tableData])

  const children = useMemo(() => {
    let children = []
    if (Array.isArray(enums)) {
      enums.forEach(v => {
        if (sourceType === '数据字典') {
          children.push(<Option value={v.nodeValue} key={i}>{v.nodeValue}</Option>)
        } else {
          let displayData = v?.find(v=> v.fieldName === displayName)
          let fieldNameData = v?.find(v=> v.fieldName === fieldName)
          children.push(<Option key={v} value={fieldNameData?.fieldValue}>{displayData?.fieldValue}</Option>)
        }
      })
    }
    return children
  }, [sourceType, enums])

 

  const onfocus = async () => {
    if (!addons) return
    if(sourceType === '数据字典') {
      getDictionary()
    }else if(!fieldTableKey){
      getTableData()
    }
  }

  const getTableData = async () => {
     if (!tableName) {
        return message.info('请选择表名!')
      }
      if (!fieldName) {
        return message.info('请选择存储字段名!')
      }
      if (!displayName) {
        return message.info('请选择展示字段名!')
      }
      const { code, data, msg } = await GetFieldValueByTableName({
        params: {
          tableName: tableName,
          fieldName: `${displayName},${fieldName}`,
        },
        data: []
      })
      if (code === 0) {
        if (Array.isArray(data)) {
          setTableData(data.filter(v => v))
        } else {
          setTableData([])
        }
      } else {
        message.error(msg)
      }
  }

  const getDictionary = async () => {
    if (!dictionary) {
      return message.error('缺少数据字典!')
    }
    const { code, data, msg } = await GetSelectItemList({ nodeName: dictionary })
    if (code === 0) {
      if (Array.isArray(data)) {
        let filterKey = null
        Object.keys(addons.formData).forEach(v => {
          if (addons.formData[v]) {
            Object.keys(addons.formData[v]).forEach(s => {
              if (fieldParent === s) {
                filterKey = addons.formData[v][s]
              }
            })
          }
        })
        if (filterKey) {
          setOptions(data.filter(v => v.nodeName === filterKey))
        } else {
          setOptions(data)
        }
      }
    } else {
      message.error(msg)
    }
  } 

  return (
    <Select
      value={value}
      onFocus={onfocus}
      style={{ width: '100%' }}
      onChange={selectChange}
      disabled={disabled}
      placeholder={disabled ? (placeholder || '') : (placeholder || '请选择内容')}
      allowClear
    >
      {
        children
      }
    </Select>
  )

}

export default RelevanceSelect